#include "StdAfx.h"
#include "UpdaterModel.h"
#include "IPC/IPCCopyData.h"
#include "TrayIconHandler.h"
#include "NotifyWindowManager.h"
#include "CheckCoreValue.h"
#include "TimerEventTrigger.h"
#include "StartConfigurationHelper.h"
#include "CheckUpgradeEventTrigger.h"
#include "NotifyWindowHandler.h"
#include "../../../common/corevalueregistryconst.h"
#include "mvc/NaverAgent/NaverAgentUtil.h"
#include "HttpClient/httpclient.h"

namespace tipupdater
{
const TCHAR APPDATA_XML_FILENAME[]				= _T("NtalkUpdater.xml");
UpdaterModel::UpdaterModel(void)
{
 	m_InstallController = NULL; 
 	m_InstallCallback = NULL;

	m_AppDataManager.LoadXmlDoc(APPDATA_XML_FILENAME);
	m_notifyWindowManager = NULL;
	m_timerEventTriggerHandler = NULL;
}

UpdaterModel::~UpdaterModel(void)
{
}

void UpdaterModel::Init()
{
	// IPC   ʱȭ
	tipbase::IPCStringKey host(_T("NtalkbizUpgrader"));
	m_hooker = new ServerHooker(GetDocument());
	m_server.SetHooker(m_hooker);
	m_server.Create(&host);

	StartConfigurationHelper helper(this);

	m_timerEventTriggerHandler = new TimerEventTriggerHandler;

#ifdef DEBUG
	const TCHAR* UPGRADER_EXE = _T("NtalkbizUpgraderLauncherD.exe"); 
#else
	const TCHAR* UPGRADER_EXE = _T("NtalkbizUpgraderLauncher.exe"); 
#endif

	

	CheckUpgradeEventTrigger* checkSelfUpgradeEventTrigger = new CheckUpgradeEventTrigger(GetDocument(), UPGRADER_EXE,  helper.GetTimeForNextShow());
	m_timerEventTriggerHandler->Add(checkSelfUpgradeEventTrigger);

	// óüũ 10еڿ ϰ,  Ĵ 1ð  Ѵ.
#ifndef _DEBUG
	if (IsTestRun())
	{
		m_timerEventTriggerHandler->Start(10 * 1, 10);
	}
	else
	{
		m_timerEventTriggerHandler->Start(60 * 10, helper.GetUpgradeCheckInterval());	
	}
#else
	m_timerEventTriggerHandler->Start(10 * 1, helper.GetUpgradeCheckInterval());
#endif


	//TrayIconHandler::CreateInstacne()->TryShow();

	NotifyWindowHandler::CreateInstance()->SetDocument(GetDocument());
	m_notifyWindowManager = new NotifyWindowManager( GetDocument() );
	m_notifyWindowManager->StartNotifyWindow(1000);

}

void UpdaterModel::Close()
{
	m_Document.Close();

	if ( m_timerEventTriggerHandler )
	{
		m_timerEventTriggerHandler->Close();
		delete m_timerEventTriggerHandler;
		m_timerEventTriggerHandler = NULL;
	}

	if ( m_notifyWindowManager)
	{
		m_notifyWindowManager->Close();
		delete m_notifyWindowManager;
		m_notifyWindowManager = NULL;
	}


	if (m_hooker)
		delete m_hooker;
	m_hooker = NULL;

	//TrayIconHandler::CreateInstacne()->_Hide();
	//TrayIconHandler::DestroyInstance();
	// [:2008-12-18] Memory Leak  : NotifyWindowHandler CreateInstance()ȣϴ ڵ尡  
	NotifyWindowHandler::DestroyInstance();
}

BOOL UpdaterModel::IsTestRun()
{
	return _waccess(_T("test.txt"), 0) == -1 ? FALSE : TRUE;
}

int ReceiveCallback::Callback(IPCPacket& packet, IIPCHandle *callback_handle )
{

	TCHAR data[MAX_PATH] = {0, };
	memcpy(data, packet.data, packet.size);

	if (_tcsicmp(_T("request_connect"), data) == 0)
	{
		TCHAR tmp[] = _T("connect_ok");
		callback_handle->Write((const BYTE*)tmp, (_tcslen(tmp)+1)*sizeof(TCHAR));
	}
	else if (_tcsicmp(_T("quit_command"), data) == 0 || _tcscmp(_T("request_isquit"), data) == 0)
	{
		if (m_Document->GetPendingService())
		{
			TCHAR tmp[] = _T("quit_wait");
			callback_handle->Write((const BYTE*)tmp, (_tcslen(tmp)+1)*sizeof(TCHAR));
		}
		else
		{
			// Quit Command
			if (_tcsicmp(_T("quit_command"), data) == 0)
			{
				callback_handle->SetReadCallback(NULL);
				PostQuitMessage(0);
			}

			TCHAR tmp[] = _T("quit_ok");
			callback_handle->Write((const BYTE*)tmp, (_tcslen(tmp)+1)*sizeof(TCHAR));
		}

	}
	else if (_tcscmp(_T("request_recreateview"), data) == 0)
	{
		Application* application = Application::get_instance();
		UpdaterView* view = dynamic_cast<UpdaterView*>(application->GetView());
		UpdaterModel* model = dynamic_cast<UpdaterModel*>(application->GetModel());
		UpdaterDocument* document = dynamic_cast<UpdaterDocument*>(model->GetDocument());
		BOOL bReload = document->Reload();
		BOOL bCreateMainWindow = view->OpenMainWindow();
		if (bReload && !bCreateMainWindow)
		{
			view->RecalcLayout();
			//view->Invalidate();
		}
	}
	packet.Release();

	return 1;
}


};